import os
from datetime import datetime
from typing import Optional

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.database import DatabaseSettings
from cpl_core.dependency_injection import ServiceProviderABC, ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironment

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.pipes.date_time_offset_pipe import DateTimeOffsetPipe
from bot_core.service.client_utils_service import ClientUtilsService
from bot_core.service.message_service import MessageService
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.db_context import DBContext
from bot_data.service.client_repository_service import ClientRepositoryService
from bot_data.service.known_user_repository_service import KnownUserRepositoryService
from bot_data.service.server_repository_service import ServerRepositoryService
from bot_data.service.user_joined_server_repository_service import UserJoinedServerRepositoryService
from bot_data.service.user_joined_voice_channel_service import UserJoinedVoiceChannelRepositoryService
from bot_data.service.user_repository_service import UserRepositoryService
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.service.base_helper_service import BaseHelperService
from modules.permission.abc.permission_service_abc import PermissionServiceABC
from modules.permission.service.permission_service import PermissionService


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)
        self._start_time = datetime.now()

        self._config: Optional[ConfigurationABC] = None

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        environment.set_working_directory(os.path.dirname(os.path.realpath(__file__)))
        configuration.add_environment_variables('KDB_')
        configuration.add_environment_variables('DISCORD_')

        configuration.add_json_file(f'config/appsettings.json', optional=False)
        configuration.add_json_file(f'config/appsettings.{environment.environment_name}.json', optional=True)
        configuration.add_json_file(f'config/appsettings.{environment.host_name}.json', optional=True)

        configuration.add_configuration('Startup_StartTime', str(self._start_time))

        self._config = configuration
        return configuration

    def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_logging()
        services.add_translation()

        services.add_db_context(DBContext, self._config.get_configuration(DatabaseSettings))

        # general services
        services.add_transient(BaseHelperABC, BaseHelperService)
        services.add_transient(MessageServiceABC, MessageService)
        services.add_transient(ClientUtilsServiceABC, ClientUtilsService)

        # pipes
        services.add_transient(DateTimeOffsetPipe)

        # module services
        services.add_singleton(PermissionServiceABC, PermissionService)

        # data services
        services.add_transient(ServerRepositoryABC, ServerRepositoryService)
        services.add_transient(UserRepositoryABC, UserRepositoryService)
        services.add_transient(ClientRepositoryABC, ClientRepositoryService)
        services.add_transient(KnownUserRepositoryABC, KnownUserRepositoryService)
        services.add_transient(UserJoinedServerRepositoryABC, UserJoinedServerRepositoryService)
        services.add_transient(UserJoinedVoiceChannelRepositoryABC, UserJoinedVoiceChannelRepositoryService)

        return services.build_service_provider()
